﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Collections;

using aSourceRewriter;
using aExtractInterface;
using aCompiler;
using aMultipleInheritance;

namespace MultipleInheritanceTest
{
	class Sample1
	{
		public static void Do()
		{
			//ソリューションのソースコードの位置を把握
			var sr = new zSourceRewriter("MultipleInheritance", "MultipleInheritanceTest\\GeneratedCode.cs");

			var usings = new[] { "using System;", "using System.Collections.Generic;", "using System.Collections.ObjectModel;" };

			//List<T>のインターフェースを抽出
			var listInterfaceCode = zExtractInterface.GetSourceCode(new zExArg
			{
				InterfaceName = "IListInterface<T>",
				Usings = usings,
				NameSpace = "Sample1",
				Type = typeof(List<>)
			});

			//ファイルに出力する
			sr.Write("Sample1_IListInterface.cs", listInterfaceCode);

			//コンパイルする。あとで別ドメインのアセンブリを読み込まずにTypeにアクセスできるTypeラッパも作ろう
			var listInterfaceAssembly = zCompiler.Compile(listInterfaceCode, new[] { "System.dll" }, "IListInterface.dll");

			var type = listInterfaceAssembly.GetType("IListInterface`1");

			//抽出したListのインターフェースを継承し、Listに委譲する。それだけだと不便なのでIList<T>も継承する。
			var listImplCode = zMultipleInheritance.GetSourceCode(new zClassData
			{
				ClassName = "ListImpl<T>",
				NameSpace = "Sample1",
				Usings = usings.Concat(new[]{ "using System.Collections;" }).ToArray(),
				//継承クラス、インターフェースを並べた後に書く部分の設定。今回はさらに継承インターフェースを書き込む。
				AfterDerives = new[]{ ", IListInterface<T>" } 
			}, new[]{
				new zInheriData{
					Type = typeof(List<>), //List<T>に委譲
					IsDerived = false, //今回行うのは委譲なので継承しない
					Name = "list", //委譲先の名前はlist
				},
				new zInheriData{
					Type = typeof(IEnumerable),
					IsDerived = true,
					Access = eAccess.InterfaceExplicit, //インターフェースの明示的実装を行う。
					Name = "(list as IEnumerable)", //明示的実装の場合フィールドをキャストしてやらないとうまくいかないことが多い
					CreateField = false //フィールドは作らない
				},
				new zInheriData{
					Type = typeof(IEnumerable<>),
					IsDerived = true,
					Access = eAccess.InterfaceExplicit,
					Name = "(list as IEnumerable<T>)",
					CreateField = false
				},
				new zInheriData{
					Type = typeof(ICollection<>),
					IsDerived = true,
					Access = eAccess.InterfaceExplicit,
					Name = "(list as ICollection<T>)",
					CreateField = false,
				},
				new zInheriData{
					Type = typeof(IList<>),
					IsDerived = true,
					Access = eAccess.InterfaceExplicit,
					Name = "(list as IList<T>)",
					CreateField = false
				}
			});

			//ファイルに出力する
			sr.Write("Sample1_ListImpl.cs", listImplCode);
		}
	}
}
