﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Windows.Forms;
using System.Reflection;

using aSourceRewriter;
using aExtractInterface;
using aMultipleInheritance;
using aCompiler;

namespace MultipleInheritanceTest
{
	class Sample2
	{
		public static void Do()
		{
			//ソリューションのソースコードの位置を把握
			var sr = new zSourceRewriter("MultipleInheritance", "MultipleInheritanceTest\\GeneratedCode.cs");

			//とりあえずControlとDictionaryを多重継承してみる。

			//多重継承というからにはprotectedメンバも使えないといけない。
			//しかし実装の多重継承は出来ない。
			//なのでprotectedメンバをいったんpublicにし、そこに委譲することで多重継承する。

			var ctlUsings = new[] { 
				"using System;", "using System.Windows.Forms;" ,"using System.Drawing;",
				"using System.Windows.Forms.Layout;","using System.ComponentModel;",
				"using System.Runtime.Remoting;"
			};

			var controlToPublic = new zInheriData
			{
				Type = typeof(Control),
				//Controlを継承する。クラスを継承するとobjectのものを除く全メンバがnewになる。
				IsDerived = true,
				//protectedメンバを使う
				UseProtected = true,
				//インスタンス名はbaseにする。
				Name = "base",
				//継承なので委譲用のフィールドは作らない
				CreateField = false,
				//MemberwiseCloneもprotectedなので使えるようにしとく。
				UseMemberwiseClone = true,
				//規定値がpublicなので別に設定する必要はない
				Access = eAccess.Public
			};

			//変なアセンブリ参照しなきゃならなくなるし使いそうにないので使わない
			controlToPublic.MemberMap["AccessibilityObject"] = new zMemberData { IsUsed = false };
			controlToPublic.MemberMap["GetAccessibilityObjectById"] = new zMemberData { IsUsed = false };
			controlToPublic.MemberMap["CreateAccessibilityInstance"] = new zMemberData { IsUsed = false };

			//古い形式
			controlToPublic.MemberMap["RenderRightToLeft"] = new zMemberData { IsUsed = false };
			//これも古い形式。ちなみにオーバーロードに対して個別に設定を行うことは現状できない
			controlToPublic.MemberMap["Scale"] = new zMemberData { IsUsed = false };

			//ので現在の形式のやつは手書きする
			controlToPublic.AfterMembers = new[]{
				"public new void Scale(SizeF factor)",
				"{",
				"	base.Scale(factor);",
				"}"};
			
			var publicControlSource = zMultipleInheritance.GetSourceCode(new zClassData
			{
				ClassName = "PublicControl",
				//パーシャルにしておく
				IsPartial = true,
				Usings = ctlUsings,
				NameSpace = "Sample2"
			}, new[]{ controlToPublic });

			//ファイルに出力する。
			sr.Write("Sample2_PublicControl.cs", publicControlSource);

			//パブリックコントロールのTypeが必要なのでコンパイルして作る
			Assembly assem = zCompiler.Compile(publicControlSource, new[] { "System.dll", "System.Windows.Forms.dll", "System.Drawing.dll" }, "PublicControl.dll");

			//GenerateInMemoryなAssemblyでは名前指定のGetTypeは出来ないらしい
			var publicControlType = assem.GetTypes().Where(type => type.Name == "PublicControl").Single();


			//あるといくらか便利なIControlを抽出			
			var ex = new zExArg
			{
				InterfaceName = "IControl",
				Usings = ctlUsings,
				NameSpace = "Sample2",
				Type = typeof(Control)
			};

			//上に合わせる
			ex.MemberMap["AccessibilityObject"] = new zExMemberArg { IsUsed = false };
			ex.MemberMap["GetAccessibilityObjectById"] = new zExMemberArg{ IsUsed = false };
			ex.MemberMap["CreateAccessibilityInstance"] = new zExMemberArg { IsUsed = false };
			ex.MemberMap["RenderRightToLeft"] = new zExMemberArg { IsUsed = false };
			ex.MemberMap["Scale"] = new zExMemberArg { IsUsed = false };

			//追加メンバを入れる。
			ex.AfterMembers = new[]{
				"void Scale(SizeF factor);"
			};

			var iControlSource = zExtractInterface.GetSourceCode(ex);

			//ファイルに出力する
			sr.Write("Sample2_IControl.cs", iControlSource);

			//Dictionaryにはprotectedメンバはない。とりあえずインターフェースを抽出する
			var dicUsings = new[] { "using System;", "using System.Collections.Generic;", "using System.Runtime.Serialization;" };

			var iDicSource = zExtractInterface.GetSourceCode(new zExArg
			{
				InterfaceName = "IDic<TKey, TValue>",
				Usings = dicUsings,
				NameSpace = "Sample2",
				Type = typeof(Dictionary<,>)
			});

			sr.Write("Sample2_DictionaryInterface.cs", iDicSource);

			//ControlとDictionaryを多重継承したインターフェースを作りたい、ので手書きする。
			//別に普通にソースコードを書けばよくて、ここに書く必要もないのだが
			sr.Write("Sample2_IControlDic.cs", @"
using System;

namespace Sample2
{
	public interface IControlDic<TKey, TValue> : IControl, IDic<TKey, TValue>{}
}");

			//コントロールのインターフェースをそのまま使い、
			//PublicControlに委譲する。
			//さらにIControlを実装し、
			//ついでにDictionaryも継承しておく

			//Controlのインターフェースを引き継ぐ
			var control = new zInheriData
			{
				Type = typeof(Control),
				//委譲先の名前はctl
				Name = "ctl",
				//フィールドは作らない
				CreateField = false,
				UseProtected = true,
				//コントロールと同じアクセシビリティに
				SameAccess = true,
				//相手がvirtualならばvirtualにする
				SameVirtual = true,
			};

			//上と合わせる
			control.MemberMap["AccessibilityObject"] = new zMemberData { IsUsed = false };
			control.MemberMap["GetAccessibilityObjectById"] = new zMemberData { IsUsed = false };
			control.MemberMap["CreateAccessibilityInstance"] = new zMemberData { IsUsed = false };
			control.MemberMap["RenderRightToLeft"] = new zMemberData { IsUsed = false };
			control.MemberMap["Scale"] = new zMemberData { IsUsed = false };

			//追加メンバを与える
			control.AfterMembers = new[]{
				"public void Scale(SizeF factor)",
				"{",
				"	ctl.Scale(factor);",
				"}"};

			var publicControlData = new zInheriData
			{
				//PublicControlに委譲する
				Type = publicControlType,
				//フィールドを作る
				CreateField = true,
				//名前をControlの委譲先と合わせる
				Name = "ctl",
				//全メンバを使わない。
				UseMember = false,
			};

			var dictionaryData = new zInheriData
			{
				//Dictionaryにも委譲する
				Type = typeof(Dictionary<,>),
				Name = "dic",
				CreateField = true
			};

			//IDictionaryも実装しとく。
			var iDictionaryData = new zInheriData
			{
				Type = typeof(IDictionary<,>),
				CreateField = false,
				IsDerived = true,
				Access = eAccess.InterfaceExplicit,
				Name = "(dic as IDictionary<TKey, TValue>)"
			};

			//ICollection<T>
			var iCollectionData = new zInheriData
			{
				Type = typeof(ICollection<>),
				CreateField = false,
				IsDerived = true,
				Access = eAccess.InterfaceExplicit,
				Name = "(dic as ICollection<KeyValuePair<TKey,TValue>>)"
			};

			//この型はICollection<T>だが、実際はICollection<KeyValuePair<TKey, TValue>>になってないといけない。
			//型名の変換を行う。
			iCollectionData.TypeNameMap.Add("T", "KeyValuePair<TKey, TValue>");

			//IEnumerable<T>
			var iEnumerableData = new zInheriData
			{
				Type = typeof(IEnumerable<>),
				CreateField = false,
				IsDerived = true,
				Access = eAccess.InterfaceExplicit,
				Name = "(dic as IEnumerable<KeyValuePair<TKey, TValue>>)"
			};

			//上と同じ
			iEnumerableData.TypeNameMap.Add("T", "KeyValuePair<TKey, TValue>");

			var iEnuData = new zInheriData
			{
				Type = typeof(System.Collections.IEnumerable),
				CreateField = false,
				IsDerived = true,
				Access = eAccess.InterfaceExplicit,
				Name = "(dic as IEnumerable)"
			};

			var ctlDicSource = zMultipleInheritance.GetSourceCode(new zClassData
			{
				ClassName = "ControlDictionary<TKey, TValue>",
				NameSpace = "Sample2",
				Usings = ctlUsings.Concat(dicUsings.Skip(1)).Concat(new[]{ "using System.Collections;"}).ToArray(),
				//IControlDicを実装する
				AfterDerives = new[]{", IControlDic<TKey, TValue>"}
			}, new[]{ control, publicControlData, dictionaryData, iDictionaryData, iCollectionData, 
				iEnumerableData, iEnuData });				
				
			sr.Write("Sample2_ControlDictionary.cs", ctlDicSource);

			//ControlとDictionaryの全メンバが使える2000行のモンスタークラスが出来上がった。
			//しかしやってることは委譲で、実際にはControlを継承しているわけじゃないのでフレームワークには入れられず、あまり意味がない。
		}
	}
}
