﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using aSourceRewriter;
using aMultipleInheritance;

namespace MultipleInheritanceTest
{
	public class Values
	{
		public string S1 { get; set; }
		public string S2;
		public string S3 { private get; set; }

		public int I1 { get; set; }
		public readonly int I2;

		//...などの膨大なメンバがあるクラスを想定
	}

	public class Values2
	{
		public string D1 { get; set; }
		public int N1 { get; set; }

		//などの膨大なメンバがあるクラスを想定
	}

	class Sample4
	{
		public static void Do()
		{
			var sr = new zSourceRewriter("MultipleInheritance", "MultipleInheritanceTest\\GeneratedCode.cs");

			var usings = new[] { "using System;", "using MultipleInheritanceTest;" };

			//インスタンス名.プロパティ名という形式でなく
			//プロパティ名だけでアクセスできるようにするため
			//名前をクラスの中に取り込む
			var valueCalcSource = zMultipleInheritance.GetSourceCode(new zClassData
			{
				//値を計算するコードを書くクラスの基本クラスを作りたい
				//値にアクセスしやすくなるように、Valuesクラスを取り込む
				ClassName = "ValueCalcBase",
				NameSpace = "Sample4",
				Usings = usings
			}, new[]{ 
                new zInheriData{
                    Name = "Values",
                    Type = typeof(Values),
                    //取り込んだものはprotectedメンバにする
                    Access = eAccess.Protected,
                    //委譲先のゲッタセッタを作り入れられるようにする
                    GetterSetter = eGetterSetter.GetSet,
                    GetterSetterAccess = eAccess.Public,
                },
                new zInheriData{
                    Name = "Values2",
                    Type = typeof(Values2),
                    Access = eAccess.Protected,
                    GetterSetter = eGetterSetter.GetSet,
                    GetterSetterAccess = eAccess.Public
                }
            });

			sr.Write("Sample4_ValueCalcBase.cs", valueCalcSource);

			//ゲッタセッタを公開したクラスからゲッタだけのクラスにすることで精神的安定を量る

			var valueGetterSource = zMultipleInheritance.GetSourceCode(new zClassData
			{
				ClassName = "ValuesGetter",
				NameSpace = "Sample4",
				Usings = usings
			}, new[]{
                new zInheriData{
                    Type = typeof(Values),
                    Name = "values",
                    //セッターを使わない
                    UseSetter = false,
                    //フィールドも使う
                    UseField = true,
                }
            });

			sr.Write("Sample4_ValuesGetter.cs", valueGetterSource);
		}
	}
}
