﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using aSourceRewriter;
using aMultipleInheritance;

namespace MultipleInheritanceTest
{
	class Sample5
	{
		public static void Do()
		{
			var sr = new zSourceRewriter("MultipleInheritance", "MultipleInheritanceTest\\GeneratedCode.cs");

			//二つのタイマーに委譲してみる

			var winTimer = new zInheriData
			{
				Type = typeof(System.Windows.Forms.Timer),
				Name = "WindowsTimer",
			};

			var threadTimer = new zInheriData
			{
				Type = typeof(System.Threading.Timer),
				Name = "ThreadTimer"
			};

			//ウインドウズの方のタイマーをWinTimerにする
			winTimer.TypeNameMap.Add("Timer", "WinTimer");

			//スレッドタイマーの方の型名をThreadTimerにする
			threadTimer.TypeNameMap.Add("Timer", "ThreadTimer");

			var doubleTimer = new zClassData
			{
				ClassName = "DoubleTimer",
				NameSpace = "Sample5",
				//usingでタイマーの名前を変える
				Usings = new[] { "using System;", "using System.Windows.Forms;", "using System.Threading;",
					"using System.ComponentModel;", "using System.Runtime.Remoting;",
					"using ThreadTimer = System.Threading.Timer;",
					"using WinTimer = System.Windows.Forms.Timer;" }
			};
			
			//両方ともMarshalByRefObjectを継承しているので3つのメンバがかぶっている。
			//どうしていいかわからないのでとりあえず両方消す。
			var m = new zMemberData { IsUsed = false };

			winTimer.MemberMap["InitializeLifetimeService"] = m;
			winTimer.MemberMap["GetLifetimeService"] = m;
			winTimer.MemberMap["CreateObjRef"] = m;
			threadTimer.MemberMap["InitializeLifetimeService"] = m;
			threadTimer.MemberMap["GetLifetimeService"] = m;
			threadTimer.MemberMap["CreateObjRef"] = m;

			//MarshalByRefObjectを継承することにする
			var marshal = new zInheriData
			{
				Type = typeof(MarshalByRefObject),
				UseMember = false,
				CreateField = false,
				IsDerived = true
			};

			//両方ともIDisposableを継承しているが、これは無視できないので
			//パーシャルクラスに実装を書く(いつものようにメンバを手書きしてもいい)
			sr.Write("Sample5_DoubleTimer2.cs", @"
using System;

namespace Sample5
{
	public partial class DoubleTimer
	{
		public void Dispose()
		{
			ThreadTimer.Dispose();
			WindowsTimer.Dispose();
		}
	}
}");
			//partialにする
			doubleTimer.IsPartial = true;

			//TimerのDisposeを消す
			winTimer.MemberMap["Dispose"] = m;
			threadTimer.MemberMap["Dispose"] = m;

			var timerSource = zMultipleInheritance.GetSourceCode(doubleTimer,
				new[] { marshal, winTimer, threadTimer, 
					//WindowsTimerがIComponentを実装してるのでやっとく
					new zInheriData {
						Type = typeof(System.ComponentModel.IComponent),
						CreateField = false,
						UseMember = false,
						IsDerived = true
					}
				});

			sr.Write("Sample5_DoubleTimer.cs", timerSource);
		}
	}
}
