﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using aSourceRewriter;
using aCommentAttributeGenerator;
using aNameSpaceReplacer;
using aMultipleInheritance;
using aExtractInterface;

namespace MultipleInheritanceTest
{
	class Sample6
	{
		public static void Do()
		{
			var sr = new zSourceRewriter("MultipleInheritanceTest", "GeneratedCode.cs");

			//VisualStudioが作る形式のコメントを、
			//委譲元にも引き継がせるサンプル

			//適当に作ったクラスのソースコードを読み込む。
			//aMultipleInheritanceをusingしているが、こうするとコメントの引継ぎが簡単。
			var source = sr.GetSourceCode("Sample6_Tekitou.cs");

			//VS形式のコメントを属性に変換
			//属性は＜summary＞を含んでいないと変換されないので注意
			source = zCommentAttributeGenerator.AppendCommentAttribute(source);

			//同じ名前だとコンパイルできないので、名前空間をGeneratedに変える
			source = zNameSpaceReplacer.ReplaceNameSpace(source, "Generated");

			//ファイルに書き出す。何をやったかは実際にこのファイルを見ないと分からないと思う。
			sr.Write("Sample6_AppendComment.cs", String.Join("\r\n", source));

			var cl = new zClassData
			{
				ClassName = "TekitouGenerated",
				NameSpace = "Sample6",
				//委譲先を複数に出来るので、classの<summary>を引き継ぐには相手を指定しないといけない
				//メンバの場合はVisualStudioComment属性が付いていれば自動的にコメントを引き継ぐ
				ClassLevelComment = typeof(Generated.Tekitou)
			};

			var i = new zInheriData
			{
				//今作ったGenerated.Tekitouに委譲
				Type = typeof(Generated.Tekitou),
				Name = "tekitou"
			};

			//コメントを置き換えたい場合、置き換えるコメントを設定
			i.MemberMap.Add("Method2", m => m.CommentData = new zCommentData
			{
				Summary = new[] { "ここのコメントは置き換える" },
				Params = new[] { "このxは使われない" }
			});

			var s = zMultipleInheritance.GetSourceCode(cl, new[]{i});

			//委譲クラスにもコメントが引き継がれた。
			sr.Write("Sample6_Generated.cs", s);

			//抽出したインターフェースにもコメントを引き継がせる

			var exArg = new zExArg
			{
				InterfaceName = "ITekitou",
				NameSpace = "Sample6",
				Type = typeof(Generated.Tekitou)
			};

			exArg.MemberMap["Method"] = new zExMemberArg
			{
				CommentData = new zCommentData
				{
					Summary =new[]{ "今回はこっちを置き換える"},
					Params = new[] { "ここのxも使われない" }
				}
			};

			s = zExtractInterface.GetSourceCode(exArg);

			//インターフェースにもコメントが引き継がれた
			sr.Write("Sample6_ITekitou.cs", s);
		}
	}
}
