﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Text.RegularExpressions;
using System.Xml.Linq;


namespace aCommentAttributeGenerator
{
	public class zCommentAttributeGenerator
	{
		/// <summary>
		/// ソースファイルのXMLコメントを属性に変えたソースコードを吐き出す。
		/// XMLとして解釈される条件は///で始まる行の連続の中に＜summary＞＜/summary＞を含むこと。
		/// /が4つ以上連続するとアウト。///の後空行が入ってもアウト。
		/// </summary>
		public static string[] AppendCommentAttribute(IList<string> source)
		{
			var list = new List<string>();
			int index = 0;

			foreach (var xml in GetXMLs(source))
			{
				Append(list, source, index, xml.EndIndex);
				AppendXml(list, xml.XML);
				index = xml.EndIndex;
			}

			Append(list, source, index, source.Count);

			return list.ToArray();
		}

		static void Append(List<string> list, IList<string> source, int start, int index)
		{
			for (int i = start; i < index; ++i)
				list.Add(source[i]);
		}

		static void AppendXml(List<string> list, string[] xml)
		{
			if (IsXml(xml))
			{
				list.Add("[VisualStudioComment(new string[]{");

				foreach (var x in xml)
				{
					list.Add("@\"///" + x.Replace("\"", "\"\"") + "\",");
				}
				list.Add("})]");
			}
		}

		/// <summary>
		/// XMLとして解釈でき＜summary＞を含むならば
		/// </summary>
		static bool IsXml(string[] xml)
		{
			XElement elem;
			try
			{
				elem = XElement.Parse("<root>" + String.Join("\r\n", xml) + "</root>");
			}
			catch { return false; }

			if (elem.Element("summary") != null)
				return true;

			return false;
		}

		class GetXMLsResult
		{
			public string[] XML;
			public int EndIndex;
		}

		/// <summary>
		/// 連続した///で始まる行をまとめて返す。
		/// きっかり3つの/の行が連続した後、空行を除く/で始まらない行に移ることが条件。
		/// </summary>
		static IEnumerable<GetXMLsResult> GetXMLs(IList<string> source)
		{
			var builder = new XMLBuilder();

			for (int i = 0; i < source.Count; ++i)
			{
				var s = source[i];
				s = s.TrimStart();

				if (s.StartsWith("/"))
				{
					if (s.StartsWith("///"))
					{
						if (s.Length >= 4 && s[3] == '/')
						{
							//4つ連続はアウト
							builder.ResetN(i);
							continue;
						}
						else
						{
							builder.Append(s.Substring(3));
							continue;
						}
					}
					else
					{
						// /で始まるけど///までいかない行。
						// 怪しいから使わずにリセット。
						builder.ResetN(i);
					}
				}
				else
				{
					var result = builder.ResetN(i);
					if (result != null)
					{
						// ///の連続の後である

						if (s == "")
						{
							// たとえ///の後でも空行が来たらアウト
							continue;
						}
						else if (s.StartsWith("[VisualStudioComment"))
						{
							//[VisualStudioCommentだったら無視する
							continue;
						}
						else
						{
							yield return result;
						}
					}
				}
			}
		}

		/// <summary>
		/// 連続した///で始まる行を保持する。
		/// </summary>
		class XMLBuilder
		{
			List<string> list = new List<string>();

			public void Append(string s)
			{
				list.Add(s);
			}

			public GetXMLsResult ResetN(int index)
			{
				if (list.Count != 0)
				{
					var r = new GetXMLsResult 
					{ 
						XML = list.ToArray(),
						EndIndex = index
					};
					list.Clear();

					return r;
				}
				return null;
			}
		}
	}
}
