﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;

using aSourceBuilder;
using aMultipleInheritanceHelpler;
using aDictionaryExtension;
using aMultipleInheritance;

namespace aExtractInterface
{
	public class zExtractInterface
	{
		/// <summary>
		/// インターフェースを抽出したソースコードを吐き出す。
		/// </summary>
		public static string GetSourceCode(zExArg arg)
		{
			try
			{
				return InnerGetSourceCode(arg);
			}
			catch (zBeforeMemberString_NumParamsException e)
			{
				throw new Exception(e.yToMessage(arg.InterfaceName));
			}
		}
		
		static string InnerGetSourceCode(zExArg arg)
		{
			var b = new zSourceBuilder();

			b.Add(arg.Usings)
				.Line("")
				.Line(
				"namespace " + arg.NameSpace).Line(
				"{").Indent(() =>
				{
					b.Add(zMultipleInheritanceHelper.BeforeMemberString(arg.CommentData, arg.Attributes, 
						arg.Type, new string[0], b));
					b.Line(
						arg.Access + (arg.Access != "" ? " " : "") + "interface " + arg.InterfaceName)
						.IndentAdd(arg.AfterInterfaceName).Line(
						"{").Indent(() =>
						{
							b.Add(arg.BeforeMembers);
							WriteMembers(arg, b);
							b.Add(arg.AfterMembers);
						}).Line(
						"}");
				}).Line(
				"}");

			return b.ToString();
		}

		static void WriteMembers(zExArg arg, zSourceBuilder b)
		{
			var type = arg.Type;
			var flags = BindingFlags.Public | BindingFlags.Instance;

			var props = type.GetProperties(flags);

			var typeMap = arg.TypeNameMap;

			foreach (var prop in props)
			{
				WriteMember(prop.Name, prop, new string[0], arg.MemberMap, b, () =>
				{
					bool get = prop.CanRead, set = prop.CanWrite;

					string getset = "{ " + (get ? "get; " : "") + (set ? "set; " : "") + "}";

					if (zMultipleInheritanceHelper.IsIndexer(prop) == false)
					{
						b.Line(TypeName(prop.PropertyType, typeMap) + " " + prop.Name + " " + getset);
					}
					else
					{
						Func<MethodInfo, string> toParamStr = m =>
						{
							var array = m.GetParameters()
								.Select(p => TypeName(p.ParameterType, typeMap) + " " + p.Name)
								.ToArray();

							return String.Join(", ", array);
						};

						var paramStr = get ? toParamStr(prop.GetGetMethod()) : toParamStr(prop.GetSetMethod());

						b.Line(TypeName(prop.PropertyType, typeMap) + " this[" + paramStr + "]" + getset);
					}
				});
			}

			b.Line("");
			var events = type.GetEvents(flags);

			foreach(var e in events)
			{
				WriteMember(e.Name, e, new string[0], arg.MemberMap, b, () =>
				{
					b.Line("event " + TypeName(e.EventHandlerType, typeMap) + " " + e.Name + ";");
				});
			}

			b.Line("");
			var methods = type.GetMethods(flags);

			foreach (var method in methods)
			{
				if (zMultipleInheritanceHelper.IsPropertyOrEvent(type, method)) continue;
				if (method.GetBaseDefinition().DeclaringType == typeof(object)) continue;

				var paramNames = method.GetParameters().Select(p => p.Name).ToArray();

				WriteMember(method.Name, method, paramNames, arg.MemberMap, b, () =>
				{
					var typeArgs = method.GetGenericArguments().Select(t => TypeName(t, typeMap)).ToArray();
					var genericParams = typeArgs.Length == 0 ? "" : "<" + String.Join(", ", typeArgs) + ">";
					var argArray = method.GetParameters().Select(p => ParamTypeName(p.ParameterType, p.IsCSharpOut(), typeMap) + " " + p.Name).ToArray();
					var args = "(" + (argArray.Length == 0 ? "" : String.Join(", ", argArray)) + ")";

					b.Line(TypeName(method.ReturnType, typeMap) + " " + method.Name + genericParams + args + ";");
				});
			}
		}

		static void WriteMember(string memberName, MemberInfo info, string[] paramNames, Dictionary<string, zExMemberArg> memberMap,
			zSourceBuilder b, Action writeMemberAction)
		{
			var member = memberMap.GetValueN(memberName);

			zCommentData comment = null;
			string[] attributes = new string[0];

			if (member != null)
			{
				if (member.IsUsed == false)
					return;

				comment = member.CommentData;
				attributes = member.Attributes;
			}

			

			try
			{
				var strArray = zMultipleInheritanceHelper.BeforeMemberString(comment, attributes, info, paramNames, b);
				if (strArray.Length != 0)
				{
					b.Line("");
					b.Add(strArray);
				}
			}
			catch (zBeforeMemberString_NumParamsException e)
			{
				e.yMemberName = memberName;
				e.yRealNumParams = paramNames.Length;
				e.yWroteNumParam = member.CommentData.Params.Length;

				throw e;
			}


			

			writeMemberAction();
		}

		static string TypeName(Type type, Dictionary<string, string> typeNameMap)
		{
			return aMultipleInheritanceHelpler.zMultipleInheritanceHelper.TypeName(type, typeNameMap);
		}

		static string ParamTypeName(Type type, bool isOut, Dictionary<string, string> typeNameMap)
		{
			return aMultipleInheritanceHelpler.zMultipleInheritanceHelper.ParamTypeName(type, isOut, typeNameMap);
		}
	}
}
