﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace aMultipleInheritance
{
	/// <summary>
	/// 委譲先の情報
	/// </summary>
	public class zInheriData
	{
		/// <summary>
		/// 委譲先の変数名。必ず設定する必要がある。
		/// </summary>
		public string Name;

		/// <summary>
		/// 委譲する型。必ず設定する必要がある。
		/// </summary>
		public Type Type;

		/// <summary>
		/// その型を継承するかどうか。規定値はfalse。
		/// </summary>
		public bool IsDerived = false;

		/// <summary>
		/// 委譲するフィールドを作るかどうか。規定値はtrue。
		/// フィールドがないと普通コンパイルエラーになるけど、インターフェースを関係ないクラスに委譲して実装したりする時に使えることも
		/// </summary>
		public bool CreateField = true;

		/// <summary>
		/// メンバを使うかどうか。falseだと全メンバを無視する。規定値はtrue
		/// </summary>
		public bool UseMember = true;

		/// <summary>
		/// 委譲の種類。private継承とかprotected継承とかみたいにも出来る。規定値はpublic
		/// </summary>
		public eAccess Access = eAccess.Public;

		/// <summary>
		/// 委譲先と同じアクセシビリティにする。Accessは無視される。規定値はfalse
		/// </summary>
		public bool SameAccess = false;

		/// <summary>
		/// オブジェクト以外のメンバで、委譲先がvirtualならvirtualにする。
		/// </summary>
		public bool SameVirtual = false;

		/// <summary>
		/// 委譲したメソッド/プロパティ/イベントをstaticにするかどうか。規定値はfalse
		/// </summary>
		public bool IsStatic = false;

		/// <summary>
		/// パブリックメンバを使うかどうか。規定値はtrue。
		/// </summary>
		public bool UsePublic = true;

		/// <summary>
		/// インターナルメンバも使うかどうか。規定値はfalse
		/// </summary>
		public bool UseInternal = false;

		/// <summary>
		/// protectedメンバも使うかどうか。規定値はfalse
		/// </summary>
		public bool UseProtected = false;

		/// <summary>
		/// フィールドも委譲するかどうか。規定値はfalse
		/// </summary>
		public bool UseField = false;

		/// <summary>
		/// 委譲先のセッターも露出するかどうか。規定値はtrue
		/// </summary>
		public bool UseSetter = true;

		/// <summary>
		/// スタティックメンバを使うかどうか。規定値はfalse
		/// </summary>
		public bool UseStatic = false;

		/// <summary>
		/// インスタンスメンバを使うかどうか。規定値はtrue
		/// </summary>
		public bool UseInstance = true;

		/// <summary>
		/// 継承元のメンバまで使うかどうか。規定値はfalse。クラスの継承ならば継承したメンバは皆出てくるが、
		/// インターフェースやstaticメンバはFlattenHierarchyをtrueにしないと出てこない。
		/// </summary>
		public bool FlattenHierarchy = false;

		/// <summary>
		/// ToStringを委譲するかどうか。規定値はfalse。
		/// </summary>
		public bool OverrideToString = false;

		/// <summary>
		/// Equals、GetHashCodeを委譲するかどうか。規定値はfalse。
		/// </summary>
		public bool OverrideEqualsAndGetHash = false;

		/// <summary>
		/// MemberwiseCloneを使うかどうか。規定値はfalse。
		/// </summary>
		public bool UseMemberwiseClone = false;

		/// <summary>
		/// 委譲先の変数のGetterやSetterを作るかどうか。規定値はNone。
		/// </summary>
		public eGetterSetter GetterSetter = eGetterSetter.None;

		/// <summary>
		/// 委譲先の変数の可視性。規定値はprivate。
		/// </summary>
		public eAccess GetterSetterAccess = eAccess.Private;

		/// <summary>
		/// メンバ定義が始まる前に追加する文字列。追加したいメンバなど入れる。規定値は何もなし。
		/// </summary>
		public string[] BeforeMembers = new string[0];


		/// <summary>
		/// メンバ定義が終わったあとに追加する文字列。追加したいメンバなど入れる。規定値は何もなし。
		/// </summary>
		public string[] AfterMembers = new string[0];

		/// <summary>
		/// メンバ個別の設定を行う。メンバ名をkeyに入れる。規定値はなにもなし。
		/// </summary>
		public zMemberMap MemberMap = new zMemberMap();

		/// <summary>
		/// 型名の変換に使う。かぶった時のusingに合わせたり、型引数Tを揃えたりする。規定値では予約語の変換が行われる。
		/// </summary>
		public Dictionary<string, string> TypeNameMap = aDefaultTypeMap.zDefaultTypeMap.Get();

		/// <summary>
		/// NameとTypeは必ず入れる必要がある。
		/// </summary>
		public zInheriData() { }
	}
}
