﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;

using aSourceBuilder;
using aMultipleInheritanceHelpler;
using aBetween;

namespace aMultipleInheritance
{
	public partial class zMultipleInheritance
	{
		public static string GetSourceCode(zClassData classData, zInheriData[] args)
		{
			try
			{
				return InnerGetSourceCode(classData, args);
			}
			catch (zBeforeMemberString_NumParamsException e)
			{
				throw new Exception(e.yToMessage(classData.ClassName));
			}
		}

		static string InnerGetSourceCode(zClassData classData, zInheriData[] args)
		{
			var c = classData;
			var b = new zSourceBuilder();

			b.Add(c.Usings)
			 .Line("")
			 .Line(
				"namespace " + c.NameSpace,
				"{")
			 .Indent(() =>
			 {
				 b.Add(zMultipleInheritanceHelper.BeforeMemberString(c.CommentData, c.Attributes, c.ClassLevelComment, new string[0], b));
				 b.Line(
					 Access(c.Access) + " " + (c.IsStatic ? "static " : "") + (c.IsPartial ? "partial " : "") + "class " + c.ClassName)
				  .Indent(() =>{
					  b.Add(Derives(args))
					   .Add(c.AfterDerives);
				  }).Line(
					 "{")
				  .Indent(() =>
				  {
					  Inner(c, args, b);
				  })
				  .Line(
					 "}");
			 })
			 .Line(
			 "}");

			return b.ToString();
		}

		static string[] Derives(zInheriData[] args)
		{
			if (args.Any(arg => arg.IsDerived) == false)
				return new string[0];

			return new[]{ ": " + String.Join(", ", args
				.Where(arg => arg.IsDerived)
				.Select(arg => TypeName(arg.Type, arg.TypeNameMap))
				.ToArray()) };
		}

		static void Inner(zClassData c, zInheriData[] args, zSourceBuilder b)
		{
			b.Add(args
				.Select(arg => arg.CreateField ? InheriField(arg) : null)
				.Where(line => line != null)
				.ToArray());

			if (c.HasConstructor)
			{
				b.Line("");

				var index = c.ClassName.IndexOf('<');
				var className = index >= 0 ? c.ClassName.Remove(index) : c.ClassName;

				var result = ConstructorArgs(args);

				var array = result.Array;

				if(result.Str.Length + className.Length <= 100)
				{
					b.Line("public " + className + "(" + result.Str + ")");
				}
				else
				{
					b.Line("public " + className + "(")
						.IndentAdd(result.Array.Append(")"));
				}
					
				b.Line(
					"{")
				 .Indent(() =>
				 {
					 b.Add(Assigns(args));
				 }).Line(
					"}");
			}

			foreach (var arg in args)
			{
				b.Add(arg.BeforeMembers);

				if (arg.UseMember)
				{
					b.Line("");
					WriteFieldProperties(arg, b);
					WriteEvents(arg, b);
					WriteMethods(arg, b);
				}

				b.Add(arg.AfterMembers);
			}
		}

		static MethodArgsResult ConstructorArgs(zInheriData[] args)
		{
			return MethodArgs(args
				.Select(data => data.CreateField ? new zMethodArgsItem(TypeName(data.Type, data.TypeNameMap), StartLower(data.Name)) : null)
				.Where(item => item != null)
				.ToArray());
		}

		struct MethodArgsResult { public string[] Array; public string Str; }

		static MethodArgsResult MethodArgs(zMethodArgsItem[] ps)
		{
			var array = ps
				.Select(p => p.Type + " " + p.Name)
				.Between(s => s + ',')
				.ToArray();

			var str = String.Join(" ", array);

			return new MethodArgsResult
			{
				Array = array,
				Str = str
			};
		}

		static string[] Assigns(zInheriData[] args)
		{
			return args
				.Select(arg => arg.CreateField ? ("this." + arg.Name + " = " + StartLower(arg.Name) + ";") : null)
				.Where(arg => arg != null)
				.ToArray();
		}

		static string StartLower(string s)
		{
			return char.ToLower(s[0]) + s.Remove(0, 1);
		}

		static string InheriField(zInheriData arg)
		{
			return Access(arg.GetterSetterAccess) + " " + TypeName(arg.Type, arg.TypeNameMap) + " " + arg.Name + GetterSetter(arg);
		}

		static string Access(eAccess access)
		{
			switch (access)
			{
				case eAccess.Public: return "public";
				case eAccess.Protected: return "protected";
				case eAccess.Private: return "private";
				case eAccess.Internal: return "internal";
				case eAccess.ProtectedInternal: return "protected internal";
				case eAccess.InterfaceExplicit: return null;
				case eAccess.None: return "";
				default: throw new Exception();
			}
		}

		static string GetterSetter(zInheriData arg)
		{
			switch (arg.GetterSetter)
			{
				case eGetterSetter.None: return ";";
				case eGetterSetter.Get: return "{ get; private set; }";
				case eGetterSetter.Set: return "{ private get; set; }";
				case eGetterSetter.GetSet: return "{ get; set; }";
				default: throw new Exception();
			}
		}

		static eAccess Access(MethodInfo info)
		{
			return Access(
				() => info.IsPublic,
				() => info.IsFamilyOrAssembly,
				() => info.IsAssembly,
				() => info.IsFamily,
				() => info.IsPrivate,
				() => info.IsFamilyAndAssembly

			);
		}

		static eAccess Access(FieldInfo info)
		{
			return Access(
				() => info.IsPublic,
				() => info.IsFamilyOrAssembly,
				() => info.IsAssembly,
				() => info.IsFamily,
				() => info.IsPrivate,
				() => info.IsFamilyAndAssembly
			);
		}

		static eAccess Access(Func<bool> isPublic, Func<bool> isFamilyOrAssembly,
			Func<bool> isAssembly, Func<bool> isFamily, Func<bool> isPrivate, Func<bool> isFamilyAndAssembly)
		{
			if (isPublic()) return eAccess.Public;
			if (isFamilyOrAssembly()) return eAccess.ProtectedInternal;
			if (isAssembly()) return eAccess.Internal;
			if (isFamily()) return eAccess.Protected;
			//FamilyAndAssemblyについてはよく分からないのでprivateにしとく
			if (isPrivate() || isFamilyAndAssembly()) return eAccess.Private;
			return eAccess.None;
		}


		static bool CanAccess(MethodInfo info, zInheriData arg)
		{
			return CanAccess(arg, Access(info));
		}

		static bool CanAccess(zInheriData arg, eAccess a)
		{
			if (a == eAccess.Public)
				return true;

			if (arg.UseInternal || arg.UseProtected && a == eAccess.ProtectedInternal)
				return true;

			if (arg.UseInternal && a == eAccess.Internal)
				return true;

			if (arg.UseProtected && a == eAccess.Protected)
				return true;

			return false;
		}

		static BindingFlags GetBinding(zInheriData arg)
		{
			return (arg.UsePublic ? BindingFlags.Public : 0) |
				(arg.UseInternal || arg.UseProtected ? BindingFlags.NonPublic : 0) |
				(arg.UseStatic ? BindingFlags.Static : 0) |
				(arg.UseInstance ? BindingFlags.Instance : 0) |
				(arg.FlattenHierarchy ? BindingFlags.FlattenHierarchy : 0);
		}

		static string[] Lines(params string[] lines) { return lines; }

	}
}
