﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;

using aSourceBuilder;
using aDictionaryExtension;

namespace aMultipleInheritance
{
	public partial class zMultipleInheritance
	{
		static void WriteFieldProperties(zInheriData arg, zSourceBuilder b)
		{
			var items = Enumerable.Range(0,0)
				.Select(i => new{ Info = default(zFieldProp), MemberData = default(zMemberData), IsIndexer = default(bool) });

			var flags = GetBinding(arg);

			if(arg.UseField)
			{			
				var fields = arg.Type.GetFields(flags);

				items = items.Concat(fields
					.Where(info => CanAccess(arg, Access(info)) && info.Name.StartsWith("<") == false)
					.Select(info =>
					{
						eGetterSetter gs;

						if (arg.UseSetter)
						{
							if (info.IsLiteral || info.IsInitOnly)
								gs = eGetterSetter.Get;
							else
								gs = eGetterSetter.GetSet;
						}
						else
						{
							gs = eGetterSetter.Get;
						}

						var m = new zMemberData
						{
							OriginalName = info.Name,
							NewName = info.Name,
							InstanceName = info.IsStatic ? TypeName(arg.Type, arg.TypeNameMap) : arg.Name,
							IsStatic = info.IsStatic,
							Access = arg.SameAccess ? Access(info) : arg.Access,
							GetterSetter = gs,
							New = arg.IsDerived && arg.Type.IsClass ? "new" : ""
						};

						return new
						{
							Info = (zFieldProp)new zFieldInfo(info),
							MemberData = m,
							IsIndexer = false
						};
					})
				);
			}
			var props = arg.Type.GetProperties(flags);

			items = items.Concat(props
				.Select(info =>
				{
					bool get, set;
					eAccess getAccess = eAccess.None, setAccess = eAccess.None;

					var getter = info.GetGetMethod(true);
					MethodInfo setter = null;

					get = getter == null ? false : CanAccess(getter, arg);

					if (get)
						getAccess = Access(getter);

					if (arg.UseSetter)
					{
						setter = info.GetSetMethod(true);

						set = setter == null ? false : CanAccess(setter, arg);

						if (set)
							setAccess = Access(setter);
					}
					else set = false;

					if (get || set)
					{
						var gs = get && set ? eGetterSetter.GetSet :
								get ? eGetterSetter.Get : eGetterSetter.Set;

						eAccess dA = eAccess.None, gA = eAccess.None, sA = eAccess.None;

						if (get && set)
						{
							Func<eAccess, int> priority = a =>
							{
								switch (a)
								{
									case eAccess.Public: return 3;
									case eAccess.ProtectedInternal: return 2;
									case eAccess.Protected: return 1;
									case eAccess.Internal: return 1;
									case eAccess.Private: return 0;
									default: throw new Exception();
								}
							};

							dA = priority(getAccess) > priority(setAccess) ? getAccess : setAccess;
							gA = dA != getAccess ? getAccess : eAccess.None;
							sA = dA != setAccess ? setAccess : eAccess.None;
						}
						else if (get)
						{
							dA = getAccess;
						}
						else
						{
							dA = setAccess;
						}

						var m = new zMemberData
						{
							OriginalName = info.Name,
							NewName = info.Name,
							InstanceName = getter.IsStatic ? TypeName(arg.Type, arg.TypeNameMap) : arg.Name,
							IsStatic = arg.IsStatic,
							Access = arg.SameAccess ? dA : arg.Access,
							GetterAccess = arg.SameAccess ? gA : eAccess.None,
							SetterAccess = arg.SameAccess ? sA : eAccess.None,
							GetterSetter = gs,
							New = arg.IsDerived && arg.Type.IsClass ? "new" :
								arg.SameVirtual && (getter ?? setter).IsVirtual ? "virtual" : ""
						};

						return new
						{
							Info = (zFieldProp)new zPropInfo(info),
							MemberData = m,
							IsIndexer = IsIndexer(info)
						};
					}
					return null;
				}).Where(item => item != null)
			);

			foreach (var item in items)
			{
				var m = item.MemberData;
				

				if (item.IsIndexer == false)
				{
					WriteMember(m.OriginalName, m, item.Info.Info, arg.MemberMap, new string[0], b, modifiedMember =>
						WriteFieldProperty(item.Info, modifiedMember, arg.TypeNameMap, b));
				}
				else
				{
					WriteMember(m.OriginalName, m, item.Info.Info, arg.MemberMap,new string[0], b, modifiedMember =>
						WriteIndexer((PropertyInfo)item.Info.Info, modifiedMember, arg.TypeNameMap, b));
				}
			}

		}

		static void WriteFieldProperty(zFieldProp info, zMemberData m, Dictionary<string, string> typeMap, zSourceBuilder b)
		{
			var access = Access(m.Access);
			//インターフェースの明示的実装の場合nullが返る
			var newName = access == null ? TypeName(info.DeclaringType, typeMap) + "." + m.NewName : m.NewName;
			access = access == null ? "": access + " ";

			var New = m.New == "" ? "" : m.New + " ";

			b.Line(
				access + (m.IsStatic ? "static " : "") + New + TypeName(info.Type, typeMap) + " " + newName).Line(
				"{")
				.Indent(() =>{
					if(m.GetterSetter == eGetterSetter.GetSet || m.GetterSetter == eGetterSetter.Get)
					{
						b.Line((m.GetterAccess != eAccess.None ? Access(m.GetterAccess) + " " : "") +
							"get{ return " + m.InstanceName + "." + m.OriginalName + "; }");
					}
					if(m.GetterSetter == eGetterSetter.GetSet || m.GetterSetter == eGetterSetter.Set)
					{
						b.Line((m.SetterAccess != eAccess.None ? Access(m.SetterAccess) + " " : "") + 
							"set{ " + m.InstanceName + "." + m.OriginalName + " = value; }");
					}
				}).Line(
				"}");
		}

		static void WriteIndexer(PropertyInfo info, zMemberData m, Dictionary<string, string> typeMap, zSourceBuilder b)
		{
			var access = Access(m.Access);
			//インターフェースの明示的実装の場合nullが返る
			var newName = access == null ? TypeName(info.DeclaringType, typeMap) + "." + "this" : "this";
			access = access == null ? "" : access + " ";

			var New = m.New == "" ? "" : m.New + " ";

			MethodInfo method;
			ParameterInfo[] paramArray;

			if (m.GetterSetter == eGetterSetter.Get || m.GetterSetter == eGetterSetter.GetSet)
			{
				method = info.GetGetMethod(true);
				paramArray = method.GetParameters();
			}
			else
			{
				//Setの場合indexerにないvalue引数が最後に出てくる
				method = info.GetSetMethod(true);
				paramArray = method.GetParameters();
				paramArray = paramArray.Take(paramArray.Length - 1).ToArray();
			}

			var parameters = String.Join(", ", paramArray
				.Select(p => TypeName(p.ParameterType, typeMap) + " " + p.Name).ToArray());

			var args = String.Join(", ", paramArray
				.Select(p => p.Name).ToArray());

			b.Line(
				access + (m.IsStatic ? "static " : "") + New + TypeName(info.PropertyType, typeMap) + " " + newName + "[" + parameters + "]").Line(
				"{")
				.Indent(() =>
				{
					if (m.GetterSetter == eGetterSetter.GetSet || m.GetterSetter == eGetterSetter.Get)
					{
						b.Line((m.GetterAccess != eAccess.None ? Access(m.GetterAccess) + " " : "") +
							"get{ return " + m.InstanceName + "[" + args + "]; }");
					}
					if (m.GetterSetter == eGetterSetter.GetSet || m.GetterSetter == eGetterSetter.Set)
					{
						b.Line((m.SetterAccess != eAccess.None ? Access(m.SetterAccess) + " " : "") +
							"set{ " + m.InstanceName + "[" + args + "] = value; }");
					}
				}).Line(
				"}");
		}

		static bool IsIndexer(PropertyInfo info)
		{
			return aMultipleInheritanceHelpler.zMultipleInheritanceHelper.IsIndexer(info);
		}
	}
}
