﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;

using aSourceBuilder;
using aDictionaryExtension;
using aMultipleInheritanceHelpler;

namespace aMultipleInheritance
{
	public partial class zMultipleInheritance
	{
		static void WriteMethods(zInheriData arg, zSourceBuilder b)
		{
			var type = arg.Type;

			var flags = GetBinding(arg);

			var methods = type.GetMethods(flags)
				.Where(m =>
				{
					
					if (m.GetBaseDefinition().DeclaringType == typeof(object))
					{
						if (arg.OverrideToString && m.Name == "ToString")
							return true;

						if (arg.OverrideEqualsAndGetHash &&
							(m.Name == "Equals" || m.Name == "GetHashCode"))
							return true;

						if (arg.UseMemberwiseClone && m.Name == "MemberwiseClone")
							return true;

						return false;
					}

					if (zMultipleInheritanceHelper.IsPropertyOrEvent(type, m))
						return false;

					return true;
				});
				

			foreach (var method in methods)
			{
				if (CanAccess(method, arg))
				{
					var typeMap = arg.TypeNameMap;
					string New;

					if (method.GetBaseDefinition().DeclaringType == typeof(object))
					{
						if (method.Name == "MemberwiseClone")
							New = "new";
						else
							New = "override";
					}
					else if (arg.IsDerived && arg.Type.IsClass)
					{
						New = "new";
					}
					else if (arg.SameVirtual && method.IsVirtual)
					{
						New = "virtual";
					}
					else
					{
						New = "";
					}

					var m = new zMemberData
					{
						OriginalName = method.Name,
						NewName = method.Name,
						InstanceName = method.IsStatic ? TypeName(type, typeMap) : arg.Name,
						IsStatic = arg.IsStatic,
						Access = arg.SameAccess ? Access(method) : arg.Access,
						New = New
					};

					WriteMember(method.Name, m, method, arg.MemberMap, 
						method.GetParameters().Select(p => p.Name).ToArray(), b,
						modifiedMember => WriteMethod(method, modifiedMember, typeMap, b));
				}
			}
		}

		static void WriteMethod(MethodInfo info, zMemberData m, Dictionary<string, string> typeMap, zSourceBuilder b)
		{
			var access = Access(m.Access);
			//インターフェースの明示的実装の場合nullが返る
			string newName = access == null ? TypeName(info.DeclaringType, typeMap) + "." + m.NewName : m.NewName;
			access = access == null ? "" : access + " ";

			var New = m.New == "" ? "" : m.New + " ";

			var genericArgs = GenericArguments(info, typeMap);

			var firstLineStart = access + (m.IsStatic ? "static " : "") + New + TypeName(info.ReturnType, typeMap) + " " + newName + genericArgs;

			var methodArgs = MethodArgs(GetMethodArgsItems(info, typeMap));

			if(firstLineStart.Length + methodArgs.Str.Length <= 110)
			{
				b.Line(firstLineStart + "(" + methodArgs.Str + ")");
			}
			else
			{
				b.Line(firstLineStart + "(")
					.IndentAdd(methodArgs.Array.Append(")"));
			}
			b.Line(
				"{")
				.Indent(() =>
				{
					b.Line(
						(IsVoid(info) ? "" : "return ") + m.InstanceName + "." + info.Name + genericArgs + "(" + Params(info) + ");"
						);
				}).Line(
				"}");
		}

		static bool IsVoid(MethodInfo info)
		{
			return info.ReturnType == typeof(void);
		}

		static string Params(MethodInfo info)
		{
			return String.Join(", ",
				info.GetParameters()
					.Select(p =>{ 
						string refOrOut = "";
						if(p.ParameterType.Name.EndsWith("&"))
						{
							if(p.IsCSharpOut())
								refOrOut = "out ";

							else
								refOrOut = "ref ";
						}
						return refOrOut + p.Name;
					})
					.ToArray());
		}

		static string GenericArguments(MethodInfo info, Dictionary<string, string> typeMap)
		{
			var array = info.GetGenericArguments()
				.Select(arg => TypeName(arg, typeMap))
				.ToArray();

			if (array.Length != 0)
			{
				return "<" + String.Join(", ", array) + ">";
			}
			return "";
		}

		static string TypeName(Type type, Dictionary<string, string> typeMap)
		{
			return aMultipleInheritanceHelpler.zMultipleInheritanceHelper.TypeName(type, typeMap);
		}

		static string ParamTypeName(Type type, bool isOut, Dictionary<string, string> typeMap)
		{
			return aMultipleInheritanceHelpler.zMultipleInheritanceHelper.ParamTypeName(type, isOut, typeMap);
		}

		static zMethodArgsItem[] GetMethodArgsItems(MethodInfo info, Dictionary<string, string> typeMap)
		{
			return info.GetParameters()
				.Select(p => new zMethodArgsItem(ParamTypeName(p.ParameterType, p.IsCSharpOut(), typeMap), p.Name))
				.ToArray();
		}
	}
}
