﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;

using aDictionaryExtension;
using aMultipleInheritance;
using aSourceBuilder;

namespace aMultipleInheritanceHelpler
{
	public class zMultipleInheritanceHelper
	{
		/// <summary>
		/// TypeからC#的に正しい型名を取り出す。型名はtypeMapで変換もする。
		/// </summary>
		public static string TypeName(Type type, Dictionary<string, string> typeMap)
		{
			return zTypeName.TypeName(type, false, typeMap);
		}

		/// <summary>
		/// C#的に正しいメソッド、コンストラクタの引数の型名を返す。typeMapで変換もする。&はref/outに変換。
		/// isOutはC#でいうoutパラメータかどうか。ParameterType.Nameに&がつかなければ無視される
		/// </summary>
		public static string ParamTypeName(Type type, bool isOut, Dictionary<string, string> typeMap)
		{
			return zTypeName.TypeName(type, isOut, typeMap);
		}

		/// <summary>
		/// メンバの前の、コメントとか属性とかの文字列を取得する。bを受け取るが書き込みはしない。
		/// </summary>
		public static string[] BeforeMemberString(
			zCommentData commentN, string[] attributes, MemberInfo infoN, string[] paramNames, zSourceBuilder b)
		{
			return zBeforeMemberString.Do(commentN, attributes, infoN, paramNames, b);
		}

		/// <summary>
		/// プロパティはget_*, set_*という名前のメソッドを自動的に作る。
		/// そのメソッドはプロパティがあるなら無視してやらないとコンパイルエラーになる。
		/// イベントの場合はadd_*,remove_*
		/// </summary>
		public static bool IsPropertyOrEvent(Type type, MethodInfo info)
		{
			bool get = info.Name.StartsWith("get_");
			bool set = info.Name.StartsWith("set_");
			if (get || set)
			{
				var prop = type.GetProperty(info.Name.Substring(4),
					BindingFlags.Public | BindingFlags.NonPublic |
					BindingFlags.Instance | BindingFlags.Static);
				
				if (prop != null)
				{
					//パラメータの数とか細かく調べていくことも出来るけどまあいいや
					return true;
				}
			}

			bool add = info.Name.StartsWith("add_");
			bool remove = info.Name.StartsWith("remove_");
			if (add || remove)
			{
				var ev = type.GetEvent(info.Name.Substring(add ? 4 : 7),
					BindingFlags.Public | BindingFlags.NonPublic |
					BindingFlags.Instance | BindingFlags.Static);

				if (ev != null)
				{
					return true;
				}
			}
			return false;
		}

		public static bool IsIndexer(PropertyInfo info)
		{
			if (info.Name == "Item")
			{
				var get = info.GetGetMethod(true);
				if (get != null && get.GetParameters().Length != 0)
					return true;

				var set = info.GetSetMethod(true);
				if (set != null && set.GetParameters().Length != 1)
					return true;
			}
			return false;
		}
	}
}
